function [He,errH,err_new]=parafac_dec(T,R,tol,complexGF,Algo,niter,elsper,Le,err_old,col,H);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 	These codes belong to the CAF_toolbox.                          	%
% 	Authors: Xavier Luciani, André L.F. De Almeida and Pierre Comon 	%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PARAFAC decomposition of tensor T for a given rank R (adapted to the CAF
% context).
% tol: Algorithm stop if the fit amelioration between successive iterations is less than
% tol.
% Algo: string specifying the algorithm: 'als' for ALESCAF, 'grad' for
% GRADCAF, 'lm' for LEMACAF or 'nlcg' for CGRADCAF
% niter is the maximal number of iteration allowed to the algorithm
% elsper is the ELS cycle, 0 means no ELS
% Le is a cell containing initial guesses of the loading matrices
% err_old is the corresponding reconstruction error
% For simulation purpose only, H is the real mixing matrix used in the simulation and to be compared with its
% estimation by ALESCAF, LEMACAF or GRADCAF. Otherwise one MUST NOT declare
% the H argument
% He is the estimated mixing matrix
% If The real mixing matrix (H) is given then errH is the NMSE between H and He
% err_new is the reconstruction error (fit of the algorithm)

errH=[];
lda=0.01;
mu= 2;
% tensor dimensions
I=size(T);
N=prod(I(2:end));
% tensor order
D=length(I);
err_vec=zeros(niter+1,1);
err_new=err_old/2;
err_rel= abs(err_new-err_old)/err_old;
err_rel_vec=zeros(niter,1);

if strcmp(Algo,'lm') %only useful with LM
    grad=construct_gradient(Le,T);
    [Hess,Jac]=construct_hessian(Le,T);
    eps1=1e-3;
    eps2=1e-12;
    flag=(max(grad)<eps1);
    nu=2;
end
if strcmp(Algo,'nlcg') %only useful with NLCG
    grad=construct_gradient(Le,T);
    dir=-grad;
    delta=dir'*dir;
    cpt1=0;
end

it=0;
Lit=cell(2,D);
Td=reshape(T,[I(1) prod(I(2:end))]);
K=cell(D,2);

% loop for estimating the component matrices
while   it<niter   && err_new >=1e-16  && err_rel>tol %&& mu> 1e-8  && lda> 1e-8 &&flag==0
    it=it+1 ;
    err_old=err_new;
    switch Algo
        case 'als'
            Lit(1,:)=Le;
            [Le,err_new]=boucle_als(Le,T);
            Lit(2,:)=Le;
            if mod(it,elsper)==0
                for d=1:D
                    K{d,1}=Lit{2,d}-Lit{1,d};
                    K{d,2}=Lit{1,d};
                end
                [p,err_new]=els(Td,K,N,R);
                for d=1:D
                    Le{d}=Lit{1,d}+p*K{d,1};
                end
            end
        case 'lm'
            if mod(it,elsper)==0
                [Le,err_new,mu,nu,Hess,grad,Jac,flag]=boucle_lm_opt4(Le,T,mu,nu,Hess,grad,Jac,eps2,eps1,flag);
            else
                [Le,err_new,mu,nu,Hess,grad,Jac,flag]=boucle_lm4(Le,T,mu,nu,Hess,grad,Jac,eps2,eps1,flag);
            end
        case 'grad'
            if mod(it,elsper)==0
                grad=construct_gradient(Le,T);
                k=1;
                for i=1:D
                    K{i,1}=-unvec(grad(k:k-1+I(i)*R),R,I(i)).';
                    k=k+I(i)*R;
                    K{i,2}=Le{i};
                end
                p=els(Td,K,N,R);
                [Le,err_new,grad]=boucle_grad_opt(Le,T,p,grad);
                lda=p;
            else
                [Le,err_new,lda]=boucle_grad(Le,T,lda);
            end
        case 'nlcg'
            [Le,err_new,dir,grad,delta,cpt1]=boucle_nlcg(Le,T,dir,grad,delta,cpt1);
    end
    clc;home;
    disp(['Cost function: ',num2str(err_new)]);
    disp(['Iterations: ',num2str(it)]);
    err_rel= abs(err_new-err_old)/err_old;
    if err_rel==0
        err_rel=inf;
    end
    err_rel_vec(it)= err_rel;
    err_vec(it)=err_new;
end

if nargin==nargin('parafac_dec');
    %remove the indeterminancy
    Lf=cell(1,D-1);
    Lv=cell(1,D-1);
    for d=2:D
        if complexGF==1
            Lf{d-1}=Le{d}(1:2:end-1,:)+sqrt(-1)*Le{d}(2:2:end,:);
        else
            Lf{d-1}=Le{d};
        end
        Lv{d-1}=H;
    end
    Lf=permscal2(Lf,Lv);
    %Mean estimation of H computed from the symmetric modes
    He=zeros(size(H));
    for d=1:D-1
        He=Lf{d}+He;
    end
    He=He/(D-1);
    %NMSE
    if complexGF==1
        errH=sum(sum(([real(H) imag(H)]-[real(He) imag(He)]).^2))/sum(sum([real(H) imag(H)].^2));
    else
        errH=sum(sum(([real(H)]-[real(He)]).^2))/sum(sum([real(H)].^2));
    end
else
    Lf=cell(1,D-1);
    for d=2:D
        if complexGF==1
            Lf{d-1}=Le{d}(1:2:end-1,:)+sqrt(-1)*Le{d}(2:2:end,:);
        else
            Lf{d-1}=Le{d};
        end
    end
    %Mean estimation of H computed from the symmetric modes
    He=zeros(size(H));
    for d=1:D-1
        He=Lf{d}+He;
    end
    He=He/(D-1);
end
figure(1000)
hold on
plot([1:it+1],log(err_vec(1:it+1)),col)
